

/*
   Main header file for FxEngine.
   FxEngine is a 3dengine using glide that works on a voodoo graphics
   compatible board, it works on voodoo graphics but voodoo2 is recommended
   if you want trilinear filtering and 800x600

   Currently supported features:
   - TRUE RGB lightning
   - TEXTURE MAPPING
   - TRILINEAR FILTERING
   - BILINEAR FILTERING
   - MIPMAPPING
   - W BUFFERING
   - UNLIMITED NUMBER OF LIGHTSOURCES
   - UNLIMITED NUMBER OF OBJECTS
   - UNLIMITED NUMBER OF POLYGONS
   - GOURAUD SHADING
*/

#include <glide.h>

// defines:

// the supported resolutions:

#define fxe640x480 0
// available on voodoo
#define fxe800x600 1
// available on voodoo2
#define fxe1024x768 2
// available on voodoo2 with SLI

// The different buffers:

#define fxeBackBuffer 0
#define fxeFrontBuffer 1

// The different fogging modes:

#define NO_FOGGING 0
#define DISTANCE_FOGGING 1

// the different detaillevels:

#define LOW_DETAIL 0     // requires voodoo
						 // 640x480 and bilinear filtered textures

#define HIGH_DETAIL 1    // requires voodoo2
						 // 800x600 and trilinear filtered textures

#define MAXIMUM_DETAIL 2 // requires voodoo2 with SLI
                         // 1024x768 and trilinear filtered textures

#define AUTOMATIC_DETAIL 3      // selects detail level automaticly
					     									// depending on hardware present

// the different shading types:
#define GOURAUD_SHADING 1
#define PHONG_SHADING 2
#define FLAT_SHADING 3
#define GOURAUD_SHADING_WSH 4   // gouraud shading with specular highlights

// the different object types:
#define OBJECT_COMPLETELY_TEXTURED 0  // Every polygon is textured in the object
#define OBJECT_NOT_TEXTURED 1         // No polygon is textured in the object
#define OBJECT_COMBO 2                // Some polygons is textured and some are not

// Maximum number of textures in world:
// (Change this definition and rebuild FxEngine
// if you want more textures)
#define MAX_NUM_TEXTURES 500

// the different texturemodes
#define TEXTURE_MAPPED 0
#define NOT_TEXTURED 1
#define BUMP_MAPPED 2		// Currently not supported
#define ENVIROMENT_MAPPED 3

// the different texture filtering modes
#define TRILINEAR_FILTERED 0
#define BILINEAR_FILTERED 1

#define FXETRUE 1
#define FXEFALSE 0

// datatypes:


typedef int FxeDetailLevel;

typedef int FxeResolution;

typedef	int fxeVertexIndex;

typedef int fxePolygonIndex;


typedef struct fxeTextureCoordinate
{
  float s,t;
} fxeTextureCoordinate;

typedef struct fxeCamera
{
  // camera position
  float x,y,z;
  // camera angle
  float ax,ay,az;
} fxeCamera;

typedef struct fxeVector
{
  float x,y,z,w;
} fxeVector;

typedef struct fxeLightsource
{
  void *Next;
  void *Prev;
  float x,y,z,r,g,b;
  // The rgb values divided by 255
  float lr,lg,lb;
  float DistanceAffection;
} fxeLightsource;

typedef struct fxeTexture
{
  void *Prev;
  void *Next;
  GrTexInfo TexInfo;
  
  int FilteringMethod;
  FxU32 VideoMemoryAddressTMU0;
  FxU32 VideoMemoryAddressTMU1;
  int DownloadedToVRAM;
  int UsedInFrame;
  void *TextureData;
} fxeTexture;

typedef fxeTexture *fxeTextureHandle;

typedef struct fxeColor
{
	float r,g,b;
} fxeColor;

// A character type - with characters you can display
// counters and text, a must in video games.
// You can make your character translucent, some cool effects is
// possible. You can also change the size of the character, enabling
// some zooming effects. The best is that this is INCREDIBLE easy to
// implement with glide.

typedef struct fxeCharacter
{
  // Handle to texture of the character bitmap
  fxeTextureHandle TextureHandle;

  // Is the character translucent
  // FXETRUE if transparent and FXEFALSE if not
  int Translucent;

  // How much translucent is this character - a value from
  // 0 to 255.0
  float AlphaLevel;

  // Is the character Transparent, this means that
  // a color in the character shouldn't be drawn.
  // FXETRUE if transparent and FXEFALSE if not.
  int Transparent;

  // The color that is transparent:
  fxeColor TransparentColor;

  // Indicates that this character is used and contains a valid character.
  int Used;

} fxeCharacter, *fxeCharacterHandle;

typedef struct fxeCharacterset
{
  fxeCharacter Characterset[256];
  float Spacing;
  float CharWidth;
  float CharHeight;
} fxeCharacterset, *fxeCharactersetHandle;


typedef struct fxeVertex
{
  float x,y,z,w;
  // In how many polygons this vertex is present:
  int NumPolys;
  // pointer to list of polygon indices of polygons that have this
  // vertex in common.
  fxePolygonIndex *PolygonIndices;
} fxeVertex;

  

typedef struct fxePolygon
{

	// If flat shading is used this is the constant color
	// of the polygon
	fxeColor ConstantColor;

  // if flat shading is used this is the flat shaded color -
  // only used internally by FxEngine
  fxeColor FlatShadedColor;

	// Pointer to list of indices into the polygons vertices
	fxeVertexIndex *VertexIndices;

	// Pointer to list of handles to clipped vertices,
	// These vertices are created by FxEngine when it clips
	// polygons to the viewing volume.
	fxeVertex *ClippedVertices;

  fxeColor *ClippedVertexColors;

	// AlphaLevel is set to the amount of transparency the polygon will have,
	// is only used if the Transparent variable is set to FXTRUE
	unsigned char AlphaLevel;


	// The handle to the texture that this polygon is using
	// (only used if TexMode is set to one of the texture alternatives.
	fxeTextureHandle TextureHandle;

	// Which texture mode the polygon is using
	// (NOT_TEXTURED,TEXTURE_MAPPED,BUMP_MAPPED,ENVIROMENT_MAPPED)
	int TexMode;

  // Pointer to list of texture coordinates
  fxeTextureCoordinate *TextureCoordinates;

	fxeTextureCoordinate *ClippedTextureCoordinates;

	// Decides if the polygon is transparent or not - if this variable is
	// set to FXETRUE then the AlphaLevel variable is used to determine
	// the level of trancparency of this polygon.
	int Transparent;

  // Is the polygon clipped (3d volume clipped)
	// This variable is only used internally by FxEngine
	int Clipped;

  // Is the polygon backface removed
  // Only used internally by FxEngine
  int Culled;

  // Is a part of the polygon clipped - used when a polygon
  // is partly clipped to the viewing frustrum.
  // When this variable is set the polygon is drawn using
  // the quDrawPolygonVertexListWithClip function
	int PartlyClipped;

  // Is a part of the polygon clipped to the NearClippingPlane.
  // If this variable is set to FXETRUE then the ClippedVertices
  // is used instead of the VertexIndices when a polygon is to
  // be rasterized. Also the NumAddedVertices is used instead of
  // NumVertices.
  int PartlyClippedZ;

  // Number of vertices including vertices added by the
  // polygon clipper
  int NumAddedVertices;

	// Is the polygon doublesided or can it be clipped
	// with backface removal
	int DoubleSided;

	// Number of vertices in polygon
	int NumVertices;

	// Normal vector to this polygon
	fxeVector Normal;

  float PolygonRadius;

} fxePolygon, *fxePolygonPtr;


typedef struct fxeObject
{
	void *Prev;
	void *Next;
  fxePolygon *PolygonList;   	 // objects polygons
	fxeVertex *VertexList;       // local vertices
	fxeColor *VertexColors;      // RGB components of every vertex
				   										 // Note! The local z value is stored as
				                       // normal z in the ooz variable not in the

	fxeVertex *VerticesWorld;  // vertices transformed to world space
	fxeVertex *VerticesCamera; // vertices transformed to camera space
  fxeVertex *ProjectedVertices;      // Coordinates projected to screenspace
                            
				   													 // - vertices ready to draw to screen
  float x,y,z;		   							 	 // position in world
	int Clipped;		   								 // Is the object clipped by FxEngine - only used internally

  float ObjectRadius;                // radius of the object
	int NumPolys;	           					 // Number of polygons in object
  int NumVertices;	   						 	 // Number of vertices in object
  int ShadingType;                   // The shadingtype the object uses
                                     // I have put the shadingtype in the
                                     // object structure instead of the
                                     // polygon structure. It's more
                                     // efficient and you probably never
                                     // want to use different
                                     // shadingtypes in a object.

  int ObjectType;                    // is the object completely texturemapped
                                     // or completely not texture mapped
                                     // or is it texture mapped and not
                                     // texturemapped. This variable is
                                     // used to increase speed as we don't
                                     // need to setup glide parameters every
                                     // polygon when rendering.
  int CompletelyVisible;             // FXETRUE if all polygons in object are
                                     // visible and FXEFALSE if not.
  float SpecConstant;
  int Transparent;
  int LowAccuracyLightning;          

} fxeObject, *fxeObjectPtr;



typedef  fxeObject *fxeObjectHandle;

typedef fxeLightsource *fxeLightsourceHandle;

// All lightsources in world
fxeLightsource *Lightsources;

// functions:
float VectorLenght(fxeVector *Vector);


void CrossProduct3d(fxeVector *Result, fxeVector *Vector1,
  fxeVector *Vector2);

// This function can be used to convert angle in degrees to and angle
// in radians.
float DegToRad(float Degree);

// This function returns the version of FxEngine. MainVersion is
// a pointer to a integer that will retrieve the mainversion of
// FxEngine. UnderVersion is a pointer to a integer that will retrieve
// the underversion of FxEngine.
void fxeGetVersion(int *MainVersion, int *UnderVersion);

// call this function first - initializes FxEngine and sets 3dfx
// videomode.
// Returns FXETRUE if successful and FXEFALSE if failure
int fxeInitialize(FxeDetailLevel DetailLevel);

// This function should be called before your application terminates
// frees memory used by FxEngine and restores videomode to videomode
// used before fxeInitialize was called.
void fxeShutdown(void);


// Camera related functions:

// Moves the camera to the position in world specified by the x,y,z
// arguments.
void fxeCameraMove(float x, float y, float z);

// Rotates the camera to the angle specified by the xa,ya,za arguments.
void fxeCameraRotate(float xa, float ya, float za);


// Ambient light related functions:

void fxeAmbientlightLight(float r, float g, float b);


// Lightsource related functions:

// To create a lightsource - call this function. It returns a handle
// to a lightsource which can be used in other lightsource related
// functions
fxeLightsourceHandle fxeLightsourceAdd(float r, float g, float b,
  float x, float y, float z, float DistanceAffection);

// Moves the lightsource with handle LightsourceHandle
// to the position x,y,z in world.
void fxeLightsourceMove(fxeLightsourceHandle LightsourceHandle,
	float x, float y, float z);

// Sets the rgb components of the lightsource with handle
// LightsourceHandle. The rgb components of the lightsource is given
// in the r,g,b arguments.
void fxeLightsourceLight(fxeLightsourceHandle LightsourceHandle,
	float r, float g, float b);

// Deletes the lightsource with handle LightsourceHandle from the world.
void fxeLightsourceDelete(fxeLightsourceHandle LightsourceHandle);


// Texture related functions:

// Loads a 3df texture file from disk, returns
// NULL if failure and handle to texture if
// successful. Returns a handle to the loaded texture.
fxeTextureHandle fxeTextureLoad(char *Filename);

// Deletes the texture with handle TextureHandle.
fxeTextureDelete(fxeTextureHandle TextureHandle);


// Object related functions:

// Loads an object in the 3de format
// from disk and positions it at 0,0,0. The 3de format is specific
// for FxEngine and such a file can be created with the 3de utility
// sent with FxEngine.
// The function returns a handle to the object.
// This handle can be used in other object related functions.
fxeObjectHandle fxeObjectLoad(char *Filename);

// Adds the object whose pointer is sent in the Object argument
// to the worlds objects.
// The function returns a handle to the object.
// This handle can be used in other object related functions.
fxeObjectHandle fxeObjectAdd(fxeObject *Object);

// Rotates a the object with handle ObjectHandle
// relative to its own coordinate system. The rotation angles is
// specified in the xa,ya,za arguments.
void fxeObjectRotate(fxeObjectHandle ObjectHandle,
  float xa, float ya, float za);

// Moves the object with handle ObjectHandle to the new position in
// world specified by the x,y,z arguments.
void fxeObjectMove(fxeObjectHandle ObjectHandle,
  float x, float y, float z);

// Scales the object with handle ObjectHandle to a new size,
// the function assumes the object is centered around its own
// coordinate system.
void fxeObjectScale(fxeObjectHandle ObjectHandle, float SFactor);

// Deletes the object with handle ObjectHandle from the world.
fxeObjectDelete(fxeObjectHandle ObjectHandle);


// Rendering associated functions:

// Renders the world. This function shall be called when the scene
// shall be rendered. The DisplayWorld parameter specifies if the
// world shall be displayed or not. FXETRUE specifies that the world
// will be displayed and FXEFALSE specifies that the world will not
// be displayed.
void fxeWorldRender(int DisplayWorld);

// This function displays the rendered World. This function needs to be called
// if DisplayWorld parameter in the call fxeWorldRender was FXEFALSE or else
// the world will not be displayed or the screen.
void fxeWorldDisplay(void);

// Character associated functions:

fxeCharactersetHandle fxeCharactersetLoad(char *FilePrefix, int Translucent,
  float AlphaLevel, int Transparent, fxeColor *TransColor, float Spacing,
  float CharWidth, float CharHeight);

void fxeCharactersetSetDefault(fxeCharactersetHandle CSH);

void fxeStringDrawXY(char *String, float X, float Y);

fxeCharacterHandle fxeCharacterCreate(fxeTextureHandle TextureHandle,
  int Translucent, float AlphaLevel, int Transparent,
  fxeColor* TransparentColor);

void fxeCharacterDraw(fxeCharacterHandle CharHandle, float X, float Y,
  float Width, float Height);

// String associated functions:


void fxeFogSetMode(int FogMode);

void fxeFogSetColor(fxeColor *FogColor);

void fxeStartFrameCount(void);

// this function returns the number of frames that have
// been rendered since last call to fxeStartFrameCount
unsigned long fxeGetFrameCount(void);

void fxeBackgroundSetColor(fxeColor *pColor);

void fxeBackgroundClear(void);
